<?php

namespace Bydedi\Security;

use Bydedi\Security\Event\UserEvent;
use Symfony\Component\EventDispatcher\EventDispatcherInterface;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\RequestStack;
use Symfony\Component\HttpFoundation\Session\SessionInterface;

/**
 * Class Authenticate
 * @package Bydedi\Security
 */
class Authenticate implements AuthenticateInterface
{
    /**
     * @var UserManagerInterface
     */
    protected $userManager;

    /**
     * @var SessionInterface
     */
    protected $session;

    /**
     * @var Request
     */
    protected $request;

    /**
     * @var UserInterface
     */
    protected $user;

    /**
     * @var EventDispatcherInterface
     */
    protected $dispatcher;

    /**
     * Authenticate constructor.
     * @param UserManagerInterface $userManager
     * @param RequestStack $requestStack
     * @param EventDispatcherInterface $dispatcher
     */
    public function __construct(
        UserManagerInterface $userManager,
        RequestStack $requestStack,
        EventDispatcherInterface $dispatcher
    ) {
        $this->userManager = $userManager;
        $this->request = $requestStack->getMasterRequest();
        if ($this->request instanceof Request) {
            $this->session = $this->request->getSession();
        }
        $this->dispatcher = $dispatcher;
        $this->user = null;
    }

    /**
     * Return current authenticated user
     *
     * @return UserInterface
     */
    public function getUser()
    {
        if (is_null($this->user) == false) {
            return $this->user;
        }

        $id = $this->session->get($this->getName());

        $user = null;

        if (!is_null($id)) {
            $user = $this->userManager->findById($id);
        }

        return $this->user = $user;
    }

    /**
     * Logs in user
     *
     * @param UserInterface $user
     * @return mixed
     */
    public function login(UserInterface $user)
    {
        $this->user = $user;

        $this->session->set($this->getName(), $user->getId());

        $rolesCodes = $user->getRolesCodes();
        $this->session->set('roles', $rolesCodes);

        $this->dispatcher->dispatch(
            SecurityEvents::USER_LOGIN,
            new UserEvent($this->getUser(), $rolesCodes, $this->request)
        );
    }

    /**
     * Authenticate user
     *
     * @param array $credentials
     * @param bool $login
     * @return bool
     */
    public function auth(array $credentials, $login = true)
    {
        $user = $this->userManager->findByLogin($credentials['login']);

        if (!is_null($user)) {
            $passwordValid = $user->validatePassword($credentials['password']);

            if ($passwordValid) {
                if ($login) {
                    $this->login($user);
                }
                return true;
            }
        }

        return false;
    }

    /**
     * Deauthenticate user
     *
     * @return void
     */
    public function logout()
    {
        if ($this->session->has($this->getName())) {
            // get the user before remove the index in session
            $user = $this->getUser();

            $rolesCode = $this->session->get('roles');
            $this->session->remove($this->getName());
            $this->session->remove('roles');

            $this->dispatcher->dispatch(
                SecurityEvents::USER_LOGOUT,
                new UserEvent($user, $rolesCode, $this->request)
            );
        }
    }

    /**
     * Unique identifier for the auth session value.
     * @return string
     */
    public function getName()
    {
        return 'user_id';
    }
}
